<?php

/**
 * @file cura_billing.ui.inc
 * Cura Billing user interface functions.
 */

/**
 * Form for generating invoices.
 */
function cura_billing_generate_invoices_form($form, &$form_state) {
  if (!isset($form_state['storage']['step'])) $form_state['storage']['step'] = 1;
//  drupal_set_message($form_state['storage']['step']);
  if (!isset($form_state['values']['method'])) $form_state['values']['method'] = 'Term';
  switch ($form_state['storage']['step']) {

    case 1:
      $notice = '<p>Make sure that the following steps have been taken before generating invoices.';
      $notice .= '<ul><li><h3>Calendar</h3><p>Check that school terms are correctly captured in the system.';
      $notice .= '<li><h3>Closures</h3><p>Make sure that all closures have been entered. This includes closures';
      $notice .= ' due to bad weather, polling days, Inset days, etc.';
      $notice .= '<li><h3>Child details</h3><p>Check that the child details are accurate especially start and';
      $notice .= ' finish dates.';
      $notice .= '<li><h3>Session plans</h3><p>The session plans for each child must be correct. In particular';
      $notice .= ' a new session plan must be entered when a child starts receiving funding.</ul>';
      $form['notice'] = array(
        '#markup' => $notice,
      );
      $form['confirm'] = array(
        '#title' => t('I confirm that the steps above have been taken'),
        '#type' => 'checkbox',
      );
      break;

    case 2:
      $notice = '<p>Select the method on which to base the billing period.</p>';
      $notice .= '<ul><li><em>Day</em> - Specify the start and finish days of the period to bill.';
      $notice .= '<li><em>Term</em> - Select the defined school terms to bill.</ul>';
      $form['notice2'] = array(
        '#markup' => $notice,
      );
      $form['method'] = array(
        '#type' => 'select',
        '#title' => t('Billing period method'),
        '#options' => array(
          'Day' => t('Day'),
          'Term' => t('Term'),
        ),
        '#default_value' => 'Term',
        '#description' => t('Select the method to set the billing period.'),
      );
      $info = field_info_field('field_child_status');
//      echo 'Field Info:<br><pre>' . print_r($info['settings']['allowed_values'], TRUE) . '</pre>';
      $form['child_statuses'] = array(
        '#type' => 'select',
        '#multiple' => TRUE,
        '#title' => t('Child statuses'),
        '#options' => $info['settings']['allowed_values'],
        '#default_value' => array('N', 'W', 'P'),
        '#required' => TRUE,
      );
      break;

    // Final step.
    case 3:
      switch ($form_state['values']['method']) {

        case 'Day':
          $notice = '<p>Set the start and finish dates for the billing period.</p>';
          $form['notice31'] = array(
            '#markup' => $notice,
          );
          $form['start'] = array(
            '#title' => t('Start'),
            '#type' => 'date_popup',
            '#date_format' => 'Y-m-d',
            '#date_label_position' => 'within',
            '#default_value' => date('Y-m-d', REQUEST_TIME),
            '#required' => TRUE,
          );
          $form['finish'] = array(
            '#title' => t('Finish'),
            '#type' => 'date_popup',
            '#date_format' => 'Y-m-d',
            '#date_label_position' => 'within',
            '#default_value' => date('Y-m-d', REQUEST_TIME),
            '#required' => TRUE,
          );
          break;

        case 'Term':
          $notice = '<p>Select the school terms to generate invoices for.</p>';
          $form['notice32'] = array(
            '#markup' => $notice,
          );
          $header = array(
            'title' => array('data' => t('Term'), 'sort' => 'asc'),
            'start' => array('data' => t('Start')),
            'finish' => array('data' => t('Finish')),
          );
          $options = cura_get_school_terms('tableselect', REQUEST_TIME, 24);
          $form['terms'] = array(
            '#title' => t('School terms'),
            '#type' => 'tableselect',
            '#header' => $header,
            '#options' => $options ? $options : array(),
            '#empty' => t('No school terms available.'),
            '#tree' => TRUE,
            '#multiple' => TRUE,
//            '#default_value' => array(),
            '#description' => t('Select the school terms to bill.'),
          );
          break;
      }
      $notice = '<p>Select the children to generate invoices for.</p>';
      $form['notice33'] = array(
        '#markup' => $notice,
      );
      $header = array(
        'name' => array('data' => t('Child'), 'field' => 'n.cura_lastname', 'sort' => 'asc'),
        'birthdate' => array('data' => t('Birthdate'), 'field' => 'n.field_birth_date'),
        'status' => array ('data' => t('Status')),
      );
      $form['children'] = array(
        '#title' => t('Children'),
        '#type' => 'tableselect',
        '#header' => $header,
        '#options' => cura_get_children('tableselect2', $form_state['values']['child_statuses']),
        '#tree' => TRUE,
        '#multiple' => TRUE,
        '#default_value' => array(),
        '#description' => t('Select the children to generate invoices for.'),
      );
      break;
//  drupal_set_message('<pre>session->children:<br>'.print_r($session->children, TRUE).'<br>'); // ** TESTING **
  }
  $form['actions'] = array('#type' => 'actions');
  if ($form_state['storage']['step'] > 1) {
    $form['actions']['previous'] = array(
      '#type' => 'submit',
      '#value' => t('Previous'),
    );
  }
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Next'),
  );
  if ($form_state['storage']['step'] == 3) $form['actions']['submit']['#value'] = t('Generate invoices');

  return $form;
}

/**
 * Submission handler for the form to generate invoices.
 */
function cura_billing_generate_invoices_form_submit($form, &$form_state) {
  if (isset($form_state['storage']['values'])) {
    $form_state['storage']['values'] = array_merge($form_state['storage']['values'], $form_state['values']);
  }
  else {
    $form_state['storage']['values'] = $form_state['values'];
  }
//  drupal_set_message('<pre>form_state-storage-values<br>'.print_r($form_state['storage']['values'], TRUE).'</pre>'); // ** TESTING **
  switch ($form_state['values']['op']) {

    case 'Next':
      $form_state['storage']['step']++;
      $form_state['rebuild'] = TRUE;
      break;

    case 'Previous':
      $form_state['storage']['step']--;
      $form_state['rebuild'] = TRUE;
      break;

    case 'Generate invoices':
      switch ($form_state['storage']['values']['method']) {

        case 'Day':
          $start = $form_state['storage']['values']['start'];
          $finish = $form_state['storage']['values']['finish'];
          break;

        case 'Term':
          $term_nids = array_keys(array_filter($form_state['storage']['values']['terms']));
          $terms = entity_load('node', $term_nids);
          $start = strtotime($terms[$term_nids[0]]->field_term_dates[LANGUAGE_NONE][0]['value']);
          $finish = strtotime($terms[$term_nids[count($terms)-1]]->field_term_dates[LANGUAGE_NONE][0]['value2']);
          break;
      }
      $child_nids = array_keys(array_filter($form_state['storage']['values']['children']));
//      drupal_set_message('<pre>Calling function cura_billing_generate_bills...</pre>'); // ** TESTING **
//      drupal_set_message('Terms:<pre>'.print_r($terms, TRUE).'</pre>');
//      drupal_set_message('<pre>Start: '.date('Y-m-d', $start).' Finish: '.date('Y-m-d', $finish).'</pre>');
      $result = cura_billing_bills_create($start, $finish, $child_nids);
      if ($result) drupal_set_message(t('Successfully generated @n invoices.', array('@n' => count($child_nids))));
      $form_state['redirect'] = 'invoices';
      break;
  }
}

/**
 * Validation handler for the form to generate invoices.
 */
function cura_billing_generate_invoices_form_validate($form, &$form_state) {
  $data = $form_state['values'];
  switch ($form_state['storage']['step']) {

    case 1:
      if (!isset($data['confirm']) || !$data['confirm']) {
        form_set_error('confirm', t('You must confirm that the steps have been taken.'));
      }
      break;

    case 2:
      break;

    case 3:
      break;
  }
}

/**
 * Wrapper for invoice form.
 *
 * @param object $invoice
 *   The cura_invoice object to be edited.
 * @param string $op
 *   The operation being performed.
 */
function cura_billing_invoice_form_wrapper($invoice, $op = 'edit') {
  return drupal_get_form('cura_billing_edit_invoice_form', $invoice, $op);
}

/**
 * Form for editing an invoice.
 */
function cura_billing_edit_invoice_form($form, &$form_state, $invoice, $op = 'edit') {
//  drupal_set_message('<pre>invoice:<br>'.print_r($invoice, TRUE).'<br>'); // ** TESTING **
  $account = $GLOBALS['user'];
  $ip = ip_address();
  $form_state['invoice'] = $invoice;
  if ($op == 'edit') {
    if ($account->uid == 1 || $invoice->status == 'Pending' || $invoice->status == 'Draft') {
      if (!isset($form_state['storage']['add_line_item'])) $form_state['storage']['add_line_item'] = FALSE;
//      drupal_set_message('<pre>invoice:<br>'.print_r($invoice, TRUE).'<br>'); // ** TESTING **
      $form['overview'] = array(
	'#markup' => t('Note that invoices can only be changed when in the pending status. '
	    .'Invoices that have been approved and posted cannot be changed.'),
      );
      $form['invoice_id'] = array(
	'#type' => 'hidden',
	'#value' => isset($invoice->invoice_id) ? $invoice->invoice_id : NULL,
      );
      $form['date'] = array(
	'#title' => t('Invoice date'),
	'#type' => 'date_popup',
	'#date_format' => 'Y-m-d',
	'#date_label_position' => 'within',
	'#default_value' => isset($invoice->date) ?
			    date('Y-m-d', $invoice->date) : date('Y-m-d', REQUEST_TIME),
	'#required' => TRUE,
      );
      $form['due_date'] = array(
	'#title' => t('Due date'),
	'#type' => 'date_popup',
	'#date_format' => 'Y-m-d',
	'#date_label_position' => 'within',
	'#default_value' => isset($invoice->due_date) ?
			    date('Y-m-d', $invoice->due_date) : date('Y-m-d', REQUEST_TIME),
	'#required' => TRUE,
      );
      $form['child_id'] = array(
	'#title' => t('Child'),
	'#type' => 'select',
	'#options' => cura_get_children('select', 'All'),
	'#default_value' => isset($invoice->child_id) ? $invoice->child_id : NULL,
	'#required' => TRUE,
      );
      $form['currency_code'] = array(
	'#title' => t('Currency code'),
	'#type' => 'select',
	'#options' => commerce_currency_code_options_list(),
	'#default_value' => isset($invoice->currency_code) ? $invoice->currency_code
			    : variable_get('commerce_default_currency', 'GBP'),
	'#required' => TRUE,
      );
      $form['description'] = array(
	'#title' => t('Description'),
	'#type' => 'textarea',
	'#default_value' => isset($invoice->description) ? $invoice->description : '',
	'#required' => TRUE,
      );
      $invoice_lines = cura_billing_get_invoice_lines($invoice->invoice_id);
      if ($form_state['storage']['add_line_item']) { // Add invoice line
	$invoice_lines[] = entity_create('cura_invoice_line', array(
			     'invoice_line_id' => NULL,
			     'label' => 'Invoice '.$invoice->invoice_id.' line '.(count($invoice_lines) + 1),
			     'invoice_id' => $invoice->invoice_id,
			     'product_id' => 0,
			     'product' => '',
			     'quantity' => 1,
			     'amount' => 0,
			     'created_by' => $account->uid,
			     'created_on' => REQUEST_TIME,
			     'created_ip' => $ip,
			     'updated_by' => $account->uid,
			     'updated_on' => REQUEST_TIME,
			     'updated_ip' => $ip,
			     'active' => 1,
			   ));
      }
//      drupal_set_message('<pre>invoice_lines:<br>'.print_r($invoice_lines, TRUE).'<br>'); // ** TESTING **
      if ($invoice_lines) {
	$product_options = db_query('SELECT product_id, title FROM {commerce_product}')->fetchAllKeyed();
	$form['invoice_lines'] = array(
	  '#markup' => t('Invoice lines'),
	  '#type' => 'container',
	  '#tree' => TRUE,
	  '#prefix' => '<table><tr><th>Product<th>Description<th>Quantity<th>Amount<tr>',
	  '#suffix' => '</table>',
	);
	foreach ($invoice_lines as $key => $invoice_line) {
//          drupal_set_message('<pre>invoice_line:<br>'.print_r($invoice_line, TRUE).'<br>'); // ** TESTING **
	  $ilw = entity_metadata_wrapper('cura_invoice_line', $invoice_line);
	  $form['invoice_lines'][$key]['invoice_line_id'] = array(
	    '#type' => 'hidden',
	    '#value' => isset($ilw->invoice_line_id) ? $ilw->invoice_line_id->value() : NULL,
	  );
	  $form['invoice_lines'][$key]['label'] = array(
	    '#type' => 'hidden',
	    '#value' => isset($ilw->label) ? $ilw->label->value() : 'Invoice I line L',
	  );
	  $form['invoice_lines'][$key]['product_id'] = array(
	    '#type' => 'select',
	    '#options' => $product_options,
	    '#default_value' => isset($ilw->product_id) ? $ilw->product_id->raw() : 0,
	    '#empty_value' => 0,
	    '#prefix' => '<tr><td>',
	    '#suffix' => '</td>',
	  );
	  $form['invoice_lines'][$key]['product'] = array(
	    '#type' => 'textfield',
	    '#size' => 60,
	    '#default_value' => isset($ilw->product) ? $ilw->product->value() : 0,
	    '#prefix' => '<td>',
	    '#suffix' => '</td>',
	  );
	  $form['invoice_lines'][$key]['quantity'] = array(
	    '#type' => 'textfield',
	    '#size' => 10,
	    '#default_value' => isset($ilw->quantity) ? $ilw->quantity->value() : 0,
	    '#prefix' => '<td>',
	    '#suffix' => '</td>',
	  );
	  $form['invoice_lines'][$key]['amount'] = array(
	    '#type' => 'textfield',
	    '#size' => 10,
	    '#default_value' => isset($ilw->amount) ? $ilw->amount->value() : 0,
	    '#prefix' => '<td>',
	    '#suffix' => '</td>',
	  );
	  field_attach_form('cura_invoice_line', $invoice_line, $form, $form_state);
	}
	$form['invoice_lines']['amount'] = array(
	  '#type' => 'markup',
	  '#markup' => t('<tr><td>Total</td><td></td><td></td><td>@amount</td>',
		       array('@amount' => isset($invoice->amount) ? $invoice->amount : 0)),
//          '#size' => 10,
//          '#default_value' => isset($invoice->amount) ? $invoice->amount : 0,
//          '#prefix' => '<tr><td>Total</td><td></td><td>',
//          '#suffix' => '</td>',
        );
      }
      $form['notes'] = array(
	'#title' => t('Notes'),
	'#type' => 'textarea',
	'#default_value' => isset($invoice->notes) ? $invoice->notes : '',
      );
      $form['status'] = array(
	'#title' => t('Status'),
	'#type' => 'textfield',
	'#size' => 60,
	'#default_value' => $invoice->status,
	'#disabled' => TRUE,
      );
      $form['active'] = array(
	'#title' => t('Active'),
	'#type' => 'checkbox',
	'#default_value' => isset($invoice->active) ? $invoice->active : 1,
      );
      $form['actions'] = array('#type' => 'actions');
      if (!$form_state['storage']['add_line_item']) {
	$form['actions']['add_line_item'] = array(
	  '#type' => 'submit',
	  '#value' => t('Add line item'),
	  '#weight' => 30,
	);
      }
      $form['actions']['submit'] = array(
	'#type' => 'submit',
	'#value' => t('Save invoice'),
	'#weight' => 40,
      );
      field_attach_form('cura_invoice', $invoice, $form, $form_state);
    }
    else {
      $form['overview'] = array(
	'#markup' => t('<p>Note that invoices can only be changed when in the pending status. '
		     .'Invoices that have been approved and posted cannot be changed.'),
      );
    }
  }
  if ($op == 'delete') {
    $form = confirm_form($form, t('Are you sure you want to delete invoice %i?', array('%i' => $invoice->invoice_id)), 'invoices', t('This action cannot be undone.'), t('Delete'), t('Cancel'));
  }

  return $form;
}

/**
 * Submission handler for the invoice form.
 */
function cura_billing_edit_invoice_form_submit($form, &$form_state) {
  $account = $GLOBALS['user'];
  $ip = ip_address();
  $invoice = $form_state['invoice'];
  $form_state['storage']['add_line_item'] = FALSE;
//  drupal_set_message('<pre>form_state->values->op:<br>'.print_r($form, TRUE).'<br>'); // ** TESTING **
  switch ($form_state['values']['op']) {
    case 'Add line item':
      $form_state['storage']['add_line_item'] = TRUE;
      $form_state['rebuild'] = TRUE;
      break;
    case 'Save invoice':
      $invoice_submission = (object) $form_state['values'];
//      drupal_set_message('<pre>invoice_submission:<br>'.print_r($invoice_submission, TRUE).'<br>'); // ** TESTING **
      $invoice_submission->date = strtotime($invoice_submission->date);
      $invoice_submission->due_date = strtotime($invoice_submission->due_date);
      $invoice_submission->tran_type = 'I';
      if (empty($invoice_submission->invoice_id)) { // New invoice
        $invoice_submission->is_new = TRUE;
        $invoice_submission->created_by = $account->uid;
        $invoice_submission->created_on = REQUEST_TIME;
        $invoice_submission->created_ip = $ip;
      }
      $invoice_submission->updated_by = $account->uid;
      $invoice_submission->updated_on = REQUEST_TIME;
      $invoice_submission->updated_ip = $ip;
      field_attach_submit('cura_invoice', $invoice_submission, $form, $form_state);
      $invoice = cura_billing_invoice_save($invoice_submission);
      $form_state['redirect'] = 'admin/invoice/'.$invoice_submission->invoice_id.'/edit';
      break;
    case 'Delete':
      cura_billing_invoice_delete($invoice->invoice_id);
      watchdog('cura_billing', 'Deleted invoice %i', array('%i' => $invoice->invoice_id));
      drupal_set_message(t('Deleted invoice %i', array('%i' => $invoice->invoice_id)));
      break;
  }
}

/**
 * Saves an invoice.
 *
 * @param object $invoice
 *   The invoice object to save.
 */
function cura_billing_invoice_save(&$invoice) {
  $account = $GLOBALS['user'];
  $ip = ip_address();
  $invoice_amount = 0;
  foreach ($invoice->invoice_lines as $invoice_line_id => $invoice_line) {
    $invoice_line['invoice_id'] = $invoice->invoice_id;
    if (empty($invoice_line['invoice_line_id'])) { // New invoice line item
      $invoice_line['created_by'] = $account->uid;
      $invoice_line['created_on'] = REQUEST_TIME;
      $invoice_line['created_ip'] = $ip;
    }
    $invoice_line['updated_by'] = $account->uid;
    $invoice_line['updated_on'] = REQUEST_TIME;
    $invoice_line['updated_ip'] = $ip;
    $ilw = entity_metadata_wrapper('cura_invoice_line', (object) $invoice_line);
    $ilw->save();
    $invoice_amount += $ilw->quantity->value() * $ilw->amount->value();
  }
//  drupal_set_message('Invoice amount: '.$invoice_amount);
  $invoice->amount = $invoice_amount;
  $iw = entity_metadata_wrapper('cura_invoice', $invoice);
  $iw->save();
}

/**
 * Returns invoices for the specified criteria.
 *
 * @param string $type
 *   The type of result to return.
 * @param $start, $finish, $child_id, $account, $status, $active
 *   The search criteria.
 *
 * @return array|false
 *   The array structure depends on $type.
 */
function cura_billing_get_invoices($type = 'id', $start, $finish, $child_id, $account, $statuses, $active) {
  $return = FALSE;
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'cura_invoice')
    ->propertyCondition('active', $active)
    ->propertyOrderBy('date', 'DESC')
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  if ($start != NULL && $finish != NULL && $finish > $start) {
    $query->propertyCondition('date', $start, '>=');
    $query->propertyCondition('date', $finish, '<=');
  }
  if ($child_id != NULL) {
    $query->propertyCondition('child_id', $child_id);
  }
  if ($account != NULL) {
    $query->propertyCondition('account', $account);
  }
  if (is_array($statuses) && $statuses[0] <> 'All') {
    $query->propertyCondition('status', $statuses);
  }
  $result = $query->execute();
  if (isset($result['cura_invoice'])) { // Found invoice entities
    $invoice_ids = array_keys($result['cura_invoice']);
    switch ($type) {
      case 'id':
        $return = $invoice_ids;
        break;
      case 'select':
        $invoices = entity_load('cura_invoice', $invoice_ids);
        foreach ($invoices as $invoice_id => $invoice) {
          $iw = entity_metadata_wrapper('cura_invoice', $invoice);
          $return[$invoice_id] = $iw->getIdentifier().' ('
                          .$iw->child_id->cura_firstname->value().' '
                          .$iw->child_id->cura_lastname->value();
        }
        break;
      case 'tableselect':
        $invoices = entity_load('cura_invoice', $invoice_ids);
        foreach ($invoices as $invoice_id => $invoice) {
          $iw = entity_metadata_wrapper('cura_invoice', $invoice);
          $return[$invoice_id] = array(
            'date' => date('Y-m-d', $iw->date->value()),
            'invoice_id' => $iw->getIdentifier(),
            'child' => $iw->child_id->cura_lastname->value().', '
                       .$iw->child_id->cura_firstname->value(),
            'amount' => $iw->amount->value(),
          );
        }
        break;
    }
//    drupal_set_message('<pre>return:<br>'.print_r($return, TRUE).'<br></pre>'); // ** TESTING **
  }

  return $return;
}

/**
 * Returns invoice lines for a specified invoice.
 */
function cura_billing_get_invoice_lines($invoice_id) {
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'cura_invoice_line')
    ->propertyCondition('invoice_id', $invoice_id)
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  $result = $query->execute();
  if (isset($result['cura_invoice_line'])) {
    $invoice_line_ids = array_keys($result['cura_invoice_line']);
//drupal_set_message('<pre>invoice_line_ids:<br>'.print_r($invoice_line_ids, TRUE).'<br>'); // ** TESTING **
    return entity_load('cura_invoice_line', $invoice_line_ids);
  }
  else {
    return FALSE;
  }
}

/**
 * Generates invoices for the children for the specified period.
 */
function cura_billing_bills_create($start, $finish, $child_nids) {
  $session_types = entity_load('cura_session_type');
  $base_bill = cura_billing_base_bill_create($start, $finish);
  $session_plans = cura_session_plan_load($start, $finish, $child_nids);
  // Check if child records have been found.
  if (!empty($child_nids)) {
    $children = node_load_multiple($child_nids);
    foreach ($children as $nid => $child) {
      $base_bill['session_plans'] = $base_bill['lines'] = array(); // Reset for each child
      foreach ($session_plans as $session_plan) {
        $spw = entity_metadata_wrapper('cura_session_plan', $session_plan);
//        echo '<pre>Commerce product:<br>'.print_r($spw->session_type_id->product_id->value()).'</pre>';
        if ($session_plan->child_id == $child->nid) {
          $base_bill['session_plans'][$session_plan->session_plan_id] = array(
            'label' => $session_plan->label,
            'session_type_id' => $spw->session_type_id->raw(),
            'session_type' => $spw->session_type_id->label->value(),
            'product_id' => $spw->session_type_id->product_id->raw(),
            'product' => $spw->session_type_id->product_id->title->value(),
            'sku' => $spw->session_type_id->product_id->sku->value(),
            'price' => $spw->session_type_id->product_id->commerce_price->amount->value(),
            'hours' => $spw->session_type_id->duration->value(),
            'sup_product_id' => $spw->session_type_id->sup_product_id->raw(),
            'sup_product' => $spw->session_type_id->sup_product_id->title->value(),
            'sup_sku' => $spw->session_type_id->sup_product_id->sku->value(),
            'sup_price' => $spw->session_type_id->sup_product_id->commerce_price->amount->value(),
            'days' => $session_plan->days,
            'funding' => $session_plan->funding,
            'start' => date('Y-m-d', $session_plan->start),
          );
          $base_bill['lines'][$spw->session_type_id->product_id->raw()] = array(
            'product_id' => $spw->session_type_id->product_id->raw(),
            'product' => $spw->session_type_id->product_id->title->value(),
            'sku' => $spw->session_type_id->product_id->sku->value(),
            'price' => $spw->session_type_id->product_id->commerce_price->amount->value(),
            'hours' => 0,
          );
          $base_bill['lines'][$spw->session_type_id->sup_product_id->raw()] = array(
            'product_id' => $spw->session_type_id->sup_product_id->raw(),
            'product' => $spw->session_type_id->sup_product_id->title->value(),
            'sku' => $spw->session_type_id->sup_product_id->sku->value(),
            'price' => $spw->session_type_id->sup_product_id->commerce_price->amount->value(),
            'hours' => 0,
          );
        }
      }
      $bill = cura_billing_bill_create($base_bill, $child);
//      drupal_set_message('<pre>bill days:<br>'.print_r($bill['days'], TRUE).'<br>'); // ** TESTING **
      cura_billing_bill_save($bill, $child, $session_types);
    }
    watchdog('cura_billing', 'Created %x bills for terms %t0.',
             array('%x' => count($children), '%t0' => implode(', ', array_keys($base_bill['terms']))));
  }
  else {
    watchdog('cura_billing', 'No children specified! Created 0 bills.');
  }
}

/**
 * Create the base bill.
 *
 * @param int $start
 *   The start of the billing period.
 * @param int $finish
 *   The finish of the billing period.
 *
 * @return array
 *   The bill initialised with the data for the billing period.
 */
function cura_billing_base_bill_create($start, $finish) {
  $calendar = cura_get_calendar($start, $finish);
  $bill = array();
  $bill['header']['date'] = date('Y-m-d', REQUEST_TIME);
  $bill['header']['due'] = date('Y-m-d', strtotime($bill['header']['date'].' +4 weeks'));
  $bill['header']['start'] = date('Y-m-d', $start);
  $bill['header']['finish'] = date('Y-m-d', $finish);
  $bill['header']['description'] = $calendar['description'];
  $bill['header']['hours_n'] = $bill['header']['hours_f'] = 0;
  $bill['header']['amount_n'] = $bill['header']['amount_f'] = 0;
  $bill['header']['currency_code'] = commerce_default_currency();
  $bill['days'] = $calendar['days'];
  $bill['terms'] = isset($calendar['terms']) ? $calendar['terms'] : NULL;

  return $bill;
}

/**
 * Generates a bill for a child for the specified period.
 */
function cura_billing_bill_create($bill, $child) {
  $account = $GLOBALS['user'];
  $ip = ip_address();
  cura_billing_bill_child_update($bill, $child);
  // Process all the session plans.
  foreach ($bill['session_plans'] as $spid => $session_plan) {
    $stid = $session_plan['session_type_id'];
    $session_type = current(entity_load('cura_session_type', array($stid)));
    $funding = explode(',', $session_plan['funding']);
    if (!isset($last[$stid])) $last[$stid] = $bill['header']['finish']; // Start at the finish
    for ($day = $last[$stid]; ($day >= $session_plan['start'] && $day >= $bill['header']['start']);
         $day = date('Y-m-d', strtotime('-1 day'.$day))) {
      if ($bill['days'][$day]['billable'] == 1) { // Day is billable
        if (strpos($session_plan['days'], date('D', strtotime($day))) !== FALSE) { // Day is in plan
          $hours_f = $funding[date('N', strtotime($day)) - 1];
          $hours_n = $session_plan['hours'] - $hours_f;
          $amount_f = $hours_f * $bill['lines'][$session_plan['sup_product_id']]['price'] / 100;
          $amount_n = $hours_n * $bill['lines'][$session_plan['product_id']]['price'] / 100;
          $bill['days'][$day]['sessions'][$stid] = array(
            'hours' => $session_plan['hours'],
            'hours_n' => $hours_n,
            'hours_f' => $hours_f,
            'amount' => $amount_n + $amount_f,
            'amount_n' => $amount_n,
            'amount_f' => $amount_f,
          );
          $bill['lines'][$session_plan['product_id']]['hours'] += $hours_n;
          $bill['lines'][$session_plan['sup_product_id']]['hours'] += $hours_f;
          $bill['days'][$day]['hours_n'] += $hours_n;
          $bill['days'][$day]['hours_f'] += $hours_f;
          $bill['header']['hours_n'] += $hours_n;
          $bill['header']['hours_f'] += $hours_f;
          $bill['header']['amount_n'] += $amount_n;
          $bill['header']['amount_f'] += $amount_f;
        }
      }
    }
    if ($session_plan['start'] < $last[$stid]) { // Make sure we are moving back
      $last[$stid] = date('Y-m-d', strtotime('-1 day'.$session_plan['start']));
    }
  }

  return $bill;
}

/**
 * Updates the child information on the bill.
 */
function cura_billing_bill_child_update(&$bill, $child) {
  $bill_start = $bill['header']['start'];
  $bill_finish = $bill['header']['finish'];
  $cw = entity_metadata_wrapper('node', $child);
  $child_start = date('Y-m-d', $cw->field_child_start_date->value());
  $child_finish = date('Y-m-d', $cw->field_child_finish_date->value());
  $bill['child'] = array(
    'nid' => $child->nid,
    'firstname' => $cw->cura_firstname->value(),
    'lastname' => $cw->cura_lastname->value(),
    'start' => $child_start,
    'finish' => $child_finish,
  );
  // Check if child started in bill period.
  if ($child_start > $bill_start && $child_start <= $bill_finish) {
    for ($day = $bill_start; $day < $child_start; $day = date('Y-m-d', strtotime('+1 day'.$day))) {
      $bill['days'][$day]['billable'] = 0;
      $bill['days'][$day]['notes'] = 'NOT STARTED';
    }
  }
  if ($child_finish < $bill_finish) { // Child left before end of bill period
    $start = ($child_finish >= $bill_start) ? date('Y-m-d', strtotime('+1 day'.$child_finish)) : $bill_start;
    for ($day = $start; $day <= $bill_finish; $day = date('Y-m-d', strtotime('+1 day'.$day))) {
      $bill['days'][$day]['billable'] = 0;
      $bill['days'][$day]['notes'] = 'LEFT';
    }
  }
}

/**
 * Save the invoice to the database.
 */
function cura_billing_bill_save($bill, $child, $session_types) {
  if ($bill['header']['hours_n'] + $bill['header']['hours_f'] > 0) {
    $account = $GLOBALS['user'];
    $ip = ip_address();

    $invoice = new stdClass();
    $invoice->label = 'Invoice X: '.$bill['child']['lastname'].', '.$bill['child']['firstname'];
    $invoice->date = strtotime($bill['header']['date']);
    $invoice->due_date = strtotime($bill['header']['due']);
    $invoice->child_id = $bill['child']['nid'];
    $invoice->description = $bill['header']['description'];
    $invoice->notes = 'Notes';
    $invoice->tran_type = 'Invoice';
    $invoice->amount = $bill['header']['amount_n'] + $bill['header']['amount_f'];
    $invoice->currency_code = $bill['header']['currency_code'];
    $invoice->status = 'Pending';
    $invoice->created_by = $account->uid;
    $invoice->created_on = REQUEST_TIME;
    $invoice->created_ip = $ip;
    $invoice->updated_by = $account->uid;
    $invoice->updated_on = REQUEST_TIME;
    $invoice->updated_ip = $ip;
    $invoice->active = 1;
    $iw = entity_metadata_wrapper('cura_invoice', $invoice);
    $iw->save();

    $line_no = 1;
    foreach ($bill['lines'] as $product_id => $line) {
      $invoice_line = new stdClass();
      $invoice_line->label = 'Invoice '.$iw->getIdentifier().' line '.$line_no;
      $invoice_line->invoice_id = $iw->getIdentifier();
      $invoice_line->product_id = $product_id;
      $invoice_line->product = $line['product'];
      $invoice_line->quantity = $line['hours'];
      $invoice_line->amount = $line['price'] / 100;
      $invoice_line->created_by = $account->uid;
      $invoice_line->created_on = REQUEST_TIME;
      $invoice_line->created_ip = $ip;
      $invoice_line->updated_by = $account->uid;
      $invoice_line->updated_on = REQUEST_TIME;
      $invoice_line->updated_ip = $ip;
      $invoice_line->active = 1;
      $ilw = entity_metadata_wrapper('cura_invoice_line', $invoice_line);
      $ilw->save();
      $line_no++;
    }

    $day_text = '<table class="table">'."\r\n"
                .'<tr><th rowspan="2">Day<th rowspan="2">Billable<th style="text-align:center" colspan="3">Hours'
                .'<th rowspan="2">Term<th rowspan="2">Notes<th rowspan="2">Closure'
                .'<th rowspan="2">Holiday</tr>'."\r\n".'<tr><th>Total<th>Non-funded<th>Funded</tr>'."\r\n";
    foreach ($bill['days'] as $key => $day) {
      $row_class = '';
      if (date('N', strtotime($key)) > 5) $row_class = ' class="active"';
      if (isset($day['closure']) || isset($day['holiday'])) $row_class = ' class="danger"';
      $hours_n = isset($day['hours_n']) ? $day['hours_n'] : 0;
      $hours_f = isset($day['hours_f']) ? $day['hours_f'] : 0;
      $hours_t = $hours_n + $hours_f;
      $notes = isset($day['notes']) ? $day['notes'] : '';
      $closure = isset($day['closure']) ? $day['closure'] : '';
      $holiday = isset($day['holiday']) ? $day['holiday'] : '';
      $day_text .= "<tr$row_class><td>".date('Y-m-d D', strtotime($key))."</td>"
                   ."<td>".$day['billable']."</td>"
                   ."<td>".$hours_t."</td><td>".$hours_n."</td><td>".$hours_f."</td>"
                   ."<td>".$day['term']."</td><td>".$notes."</td><td>".$closure."</td>"
                   ."<td>".$holiday."</td></tr>\r\n";

      // Create billing records.
      if (!empty($day['sessions'])) {
        foreach ($day['sessions'] as $session_type_id => $billing_data) {
          $session_type = $session_types[$session_type_id];
          $billing_data['invoice_id'] = $invoice->invoice_id;
          if (!cura_billing_billing_new($session_type, strtotime($key), $child, $billing_data)) {
            watchdog('cura_billing', 'Could not create billing record for session %s.',
              array('%s' => $session->session_id));
          }
        }
      }
    }
    $day_text .= '</table>';

    $invoice->label = 'Invoice '.$iw->getIdentifier().': '.$bill['child']['lastname'].', '.$bill['child']['firstname'];
    $invoice->notes = $day_text;
    $iw->save();
  }
}

///**
// * Save the commerce order to the database.
// */
//function cura_billing_commerce_order_save($invoice) {
//  $uid = 105;
//  $product_1_id = 5;
//  $product_2_id = 6;
//  $order = commerce_order_new($uid, 'checkout_review');
//  commerce_order_save($order);
//  $product_1 = commerce_product_load($product_1_id);
//  $line_item_1 = commerce_product_line_item_new($product_1, $bill['qtys']['n_sessions'], $order->order_id);
//  commerce_line_item_save($line_item_1);
//  $product_2 = commerce_product_load($product_2_id);
//  $line_item_2 = commerce_product_line_item_new($product_2, $bill['qtys']['f_sessions'], $order->order_id);
//  commerce_line_item_save($line_item_2);
//
//  // Add line items to order.
//  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
//  $order_wrapper->commerce_line_items[] = $line_item_1;
//  $order_wrapper->commerce_line_items[] = $line_item_2;
//  commerce_order_save($order);
//}

/******************************************************************************
 *                                                                            *
 * Post invoices                                                              *
 *                                                                            *
 ******************************************************************************/

/**
 * Form for posting invoices.
 */
function cura_billing_post_invoices_form($form, &$form_state) {
  $start = isset($form['storage']['filter']['start']) ? $form['storage']['filter']['start'] : NULL;
  $finish = isset($form['storage']['filter']['finish']) ? $form['storage']['filter']['finish'] : NULL;
  $child_id = isset($form['storage']['filter']['child_id']) ? $form['storage']['filter']['child_id'] : NULL;

  // Display the notice.
  $notice = '<p>The following actions will be taken when posting invoices:</p>';
  $notice .= '<ul><li><h3>Orders</h3><p>Corresponding orders will be created.';
  $notice .= '<li><h3>Sessions</h3><p>Records will be generated for the billed sessions.';
  $notice .= '<li><h3>Invoice status</h3><p>The status of the invoice will be set to posted and it will ';
  $notice .= 'no longer be possible to change the invoice.</ul>';
  $form['notice'] = array(
    '#markup' => $notice,
  );

  // Filter section.
  $form['filter'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter criteria'),
    '#collapsible' => TRUE,
  );
  $form['filter']['start'] = array(
    '#title' => t('Start date'),
    '#type' => 'date_popup',
    '#date_format' => 'Y-m-d',
    '#date_label_position' => 'within',
    '#default_value' => $start,
    '#prefix' => '<div class="col-xs-12 col-sm-4">',
    '#suffix' => '</div>',
  );
  $form['filter']['finish'] = array(
    '#title' => t('Finish date'),
    '#type' => 'date_popup',
    '#date_format' => 'Y-m-d',
    '#date_label_position' => 'within',
    '#default_value' => $finish,
    '#prefix' => '<div class="col-xs-12 col-sm-4">',
    '#suffix' => '</div>',
  );
  $form['filter']['actions'] = array('#type' => 'actions');
  $form['filter']['actions']['filter'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
  );

  // Data entry section.
  $header = array(
    'date' => array('data' => t('Date')),
    'invoice_id' => array('data' => t('Invoice #')),
    'child' => array('data' => t('Child')),
    'amount' => array ('data' => t('Amount')),
  );
  $statuses = array('Pending');
  $options = cura_billing_get_invoices('tableselect', $start, $finish, $child_id, NULL, $statuses, TRUE);
  if ($options) { // Invoices to post
    $form['invoices'] = array(
      '#title' => t('Invoices'),
      '#type' => 'tableselect',
      '#header' => $header,
      '#options' => $options,
      '#tree' => TRUE,
      '#multiple' => TRUE,
      '#default_value' => array(),
      '#description' => t('Select the invoices to post.'),
    );
  }
  else {
    $form['invoices'] = array(
      '#markup' => t('<p>No invoices to post.</p>'),
    );
  }
//  drupal_set_message('<pre>session->children:<br>'.print_r($session->children, TRUE).'<br>'); // ** TESTING **
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Post invoices'),
  );

  return $form;
}

/**
 * Submission handler for the form to post invoices.
 */
function cura_billing_post_invoices_form_submit($form, &$form_state) {
//  if (isset($form_state['values']['filter'])) {
    $form_state['storage']['filter'] = $form_state['values']['filter'];
//  }
//  drupal_set_message('<pre>form_state-storage-values<br>'.print_r($form_state['storage']['values'], TRUE).'</pre>'); // ** TESTING **
  switch ($form_state['values']['op']) {

    case 'Filter':
      $form_state['rebuild'] = TRUE;
      break;

    case 'Post invoices':
      $invoice_ids = array_keys(array_filter($form_state['values']['invoices']));
      drupal_set_message('<pre>Posting invoices...<br>'.print_r($invoice_ids, TRUE).'</pre>'); // ** TESTING **
//      drupal_set_message('Terms:<pre>'.print_r($terms, TRUE).'</pre>');
      cura_billing_post_invoices($invoice_ids);
      $form_state['rebuild'] = TRUE;
      break;
  }
}

/**
 * Posts specified invoices.
 *
 * @param array $invoice_ids
 *   The IDs of the invoices to post.
 */
function cura_billing_post_invoices($invoice_ids) {
  if ($invoices = entity_load('cura_invoice', $invoice_ids)) {
    foreach ($invoices as $invoice_id => $invoice) {
      $iw = entity_metadata_wrapper('cura_invoice', $invoice);
      if (cura_billing_create_order_from_invoice($invoice)) {
        if ($billings = cura_billing_load_by_invoice($invoice->invoice_id)) {
          // Make billing and sessions active.
          foreach ($billings as $billing_id => $billing) {
            $billing->status = 'Posted';
            $billing->active = TRUE;
            entity_save('cura_billing', $billing);
            $session = current(entity_load('cura_session', (array) $billing->session_id));
            $session->active = TRUE;
            entity_save('cura_session', $session);
          }
        }
        $invoice->date = REQUEST_TIME;
        $invoice->due_date = strtotime('+4 weeks', REQUEST_TIME);
        $invoice->status = 'Posted';
        entity_save('cura_invoice', $invoice);

        // Bill notification.
        $child_name = $iw->child_id->cura_firstname->value().' '.$iw->child_id->cura_lastname->value();
        $notification = new stdClass();
        $notification->label = $invoice->label;
        $notification->type = 'Invoice';
        $notification->notes = 'Invoice '.$iw->getIdentifier().' for '.$child_name.' for the period '
          .$iw->description->value().'.';
        $notification->date = $invoice->date;
        $notification->child_id = $iw->child_id->getIdentifier();
        $notification->uid = $iw->child_id->field_account_id->field_account_owner->field_user_id->raw();
        cura_notification_save($notification);

        // Email invoice.
        cura_billing_send_invoice_email($invoice);

      }
      else {
        watchdog('cura_billing', 'Could not create order for invoice %i.', array('%i' => $invoice_id));
      }
    }
  }
}

/**
 * Creates a new commerce order from the invoice.
 *
 * @param object $invoice
 *   The cura_invoice object from which to create the order.
 * @param string $status
 *   Status of the order.
 *
 * @return object|false
 *   A commerce_order object in the given status.
 */
function cura_billing_create_order_from_invoice($invoice, $status = 'checkout_review') {
  $order = FALSE;
  $uid = NULL;
  $iw = entity_metadata_wrapper('cura_invoice', $invoice);
  $person_id = $iw->child_id->field_account_id->field_account_owner->raw();
  $child_name = $iw->child_id->cura_firstname->value().' '.$iw->child_id->cura_lastname->value();
  if ($person = node_load($person_id)) {
    if (isset($person->field_user_id[LANGUAGE_NONE][0]['target_id'])) {
      $uid = $person->field_user_id[LANGUAGE_NONE][0]['target_id'];
    }
    // No user ID for person.
    else {
      $name = $person->field_firstname[LANGUAGE_NONE][0]['value'] . ' ' . $person->field_lastname[LANGUAGE_NONE][0]['value'];
      // Check if there is already a user with this name.
      if (!$user = user_load_by_name($name)) {
        $user = entity_create('user', array(
          'name' => $name,
          'mail' => $person->field_email[LANGUAGE_NONE][0]['email'],
          'status' => 1,
        ));
        entity_save('user', $user);
      }
      $uid = $user->uid;
      if ($user->mail == $person->field_email[LANGUAGE_NONE][0]['email']) {
        $person->field_user_id[LANGUAGE_NONE][0]['target_id'] = $uid;
        entity_save('node', $person);
      }
    }
    module_load_include('inc', 'cura', 'cura.admissions');
    $profile = cura_billing_customer_profile_new('billing', $person);
//    drupal_set_message('<pre><br>iw:<br>'.print_r($iw, TRUE).'<br>'); // ** TESTING **
    $order = commerce_order_new($uid, $status, 'commerce_order');
//    $vid = db_query('SELECT vid FROM {taxonomy_vocabulary} WHERE machine_name = :machine_name',
//      array(':machine_name' => 'order_type'))->fetchField();
    $term = cura_get_term_by_name('Hours', 'order_type');
    $order->field_order_type[LANGUAGE_NONE][0]['tid'] = $term->tid;
    if (!empty($profile->profile_id)) {
      $order->commerce_customer_billing[LANGUAGE_NONE][0]['profile_id'] = $profile->profile_id;
    }
    $order->data['reference'] = $child_name;
    $order->data['invoice_id'] = $iw->getIdentifier();
    $order->log = t('<a href="/invoice/@i/?destination=invoices">Invoice @i</a>', array('@i' => $iw->getIdentifier()));
    commerce_order_save($order);
    $ow = entity_metadata_wrapper('commerce_order', $order);
    if ($invoice_lines = cura_billing_get_invoice_lines($iw->getIdentifier())) {
      foreach ($invoice_lines as $invoice_line_id => $invoice_line) {
        if (($product = commerce_product_load($invoice_line->product_id)) && $invoice_line->quantity > 0) {
          $line_item = commerce_product_line_item_new($product, $invoice_line->quantity, $order->order_id);
          commerce_line_item_save($line_item);
          $ow->commerce_line_items[] = $line_item;
        }
      }
    }
    commerce_order_save($order);
    // Save the order ID on the invoice.
    $iw->order_id = $ow->getIdentifier();
    $iw->save();
  }

  return $order;
}

/**
 * Emails the invoice to the owner.
 *
 * @param object $invoice
 *   The cura_invoice object from which to create the email.
 *
 * @return true|false
 *   True if the email is sent.
 */
function cura_billing_send_invoice_email($invoice) {

  $module = 'cura_billing';
  $key = 'invoice';
  $language = language_default();
  $send = TRUE;

  // Get the relevant settings.
  $settings = cura_get_settings('general');
  $email_settings = cura_get_email_settings($key);

  // Check that email notifications have been enabled.
  if (!$settings['general']['notification']['email']['enabled']) {
    watchdog($module, 'Email notifications are not enabled!');
    return;
  }
  if (!$email_settings['enabled']) {
    watchdog($module, 'Invoice email notifications are not enabled!');
    return;
  }

  $org_name = variable_get('cura_org_name', '-- Set the organisation name --');
  $iw = entity_metadata_wrapper('cura_invoice', $invoice);
  $person = $iw->child_id->field_account_id->field_account_owner->value();
  $pw = entity_metadata_wrapper('node', $person);
  $title = $person->field_person_title[LANGUAGE_NONE][0]['value'];
  $owner = $person->field_firstname[LANGUAGE_NONE][0]['value'] . ' ' . $person->field_lastname[LANGUAGE_NONE][0]['value'];
  $countries = country_get_list();
  $country = $countries[$person->field_postal_address[LANGUAGE_NONE][0]['country']];
  $address = $person->field_physical_address[LANGUAGE_NONE][0]['thoroughfare'] . '<br>';
  $address .= $person->field_physical_address[LANGUAGE_NONE][0]['locality'] . ', ';
  $address .= $person->field_physical_address[LANGUAGE_NONE][0]['administrative_area'] . ' ';
  $address .= $person->field_physical_address[LANGUAGE_NONE][0]['postal_code'] . '<br>' . $country;
  $email = $person->field_email[LANGUAGE_NONE][0]['email'];
  $mobile = $pw->field_mobile->value();
  $user_url = file_create_url('user');
  $date = date('D, j M Y', $iw->date->value());
  $due_date = date('D, j M Y', $iw->due_date->value());
  $acc_no = $iw->child_id->field_account_id->field_account_number->raw();
  $child_name = $iw->child_id->cura_firstname->value() . ' ' . $iw->child_id->cura_lastname->value();
  $child_id = $iw->child_id->raw();
  $start = date('D, j M Y', $iw->child_id->field_child_start_date->value());
  $finish = date('D, j M Y', $iw->child_id->field_child_finish_date->value());
  $desc = check_markup($iw->description->value());
  $total = 0;
  $from = $email_settings['from'];
  $to = $owner . ' <' . $email . '>';
//  $to = 'Administrator <' . variable_get('site_mail', ini_get('sendmail_from')) . '>'; // ** TESTING **
  $body = '<table style="width:100%;padding:10px;">';
  $body .= '<tr><td>' . $title . ' ' . $owner . '<br>' . $address . '<br><br>' . $email . '<br>' . $mobile;
  $body .= '<td>Invoice: ' . $iw->getIdentifier() . '<br>Date: ' . $date . '<br>Due: ' . $due_date;
  $body .= '<br>Account: <a href="' . $user_url . '">' . $acc_no . '</a>';
  $body .= '<br>Child: ' . $child_name . ' (' . $child_id . ')<br>Start: ' . $start . '<br>Finish: ' . $finish;
  $body .= '</table>';
  $body .= '<table style="width:100%;padding:10px;">';
  $body .= '<tr><td style="padding:0 10px 0 10px;border:1px solid #e5e5e5;">' . $desc . '</table>';
  $body .= '<table style="width:100%;padding:10px;">';
  $body .= '<tr><th style="padding-bottom:10px;text-align:left">Product';
  $body .= '<th style="padding-bottom:10px;text-align:right">Qty';
  $body .= '<th style="padding-bottom:10px;text-align:right">Amount';
  $body .= '<th style="padding-bottom:10px;text-align:right">Total';
  if ($invoice_lines = cura_billing_get_invoice_lines($iw->getIdentifier())) {
    foreach ($invoice_lines as $line) {
      $ilw = entity_metadata_wrapper('cura_invoice_line', $line);
      $amount = commerce_currency_format($ilw->amount->value() * 100, $iw->currency_code->value());
      $subtotal = commerce_currency_format($ilw->quantity->value() * $ilw->amount->value() * 100, $iw->currency_code->value());
      $body .= '<tr><td>' . $ilw->product->value();
      $body .= '<td style="text-align:right">' . $ilw->quantity->value();
      $body .= '<td style="text-align:right">' . $amount;
      $body .= '<td style="text-align:right">' . $subtotal;
      $total += $ilw->quantity->value() * $ilw->amount->value() * 100;
    }
  }
  else {
    $body .= '<tr><td colspan="4">No invoice lines.';
  }
  $total = commerce_currency_format($total, $iw->currency_code->value());
  $body .= '<tr><td style="padding-top:10px;" colspan="3"><b>Total</b>';
  $body .= '<td style="padding-top:10px;text-align:right;"><b>'.$total.'</b></table>';
  $body .= '<div style="font-size:0.85em">';
  $body .= '<p>A late payment fee of ' . cura_product_price_get(variable_get('cura_late_product_id', '0'));
  $body .= ' will be charged if payment is not received on or before the due date.</p>';
  $body .= '<p>Please pay by electronic bank transfer if at all possible. Our bank account details are HSBC,';
  $body .= ' Summertown branch, account number 71206338, sort code 40-35-35 and remember to provide the name';
  $body .= ' of your child as the payment reference. Please make cheques payable to <i>' . check_plain($org_name) . '</i>';
  $body .= ' and write the name of your child on the back.</p>';
  $body .= '<p>Contact us if any of the information on this invoice is incorrect.</p>';
  $body .= '<p>Also note that we require a full school term notice if you want to reduce the amount of sessions';
  $body .= ' your child attends. That is, if a term has already started we will only be able to reduce your';
  $body .= ' sessions from the end of the following term.</p>';
  $body .= '</div>';
  $params = array(
    'headers' => array(
      'Cc' => $email_settings['cc'],
    ),
    'subject' => t('Invoice @i', array('@i' => $iw->getIdentifier())),
    'body' => array($body),
  );
  $result = cura_send_email($module, $key, $to, $language, $params, $from, $send);
  if ($result['result'] == true) {
    watchdog($module, 'Emailed invoice to @to.', array('@to' => $to));
  }
  else {
    watchdog($module, 'Could not email invoice to @to.', array('@to' => $to));
  }

  return $result;
}

